
/*************************************************************
Module		: Init Test module
*************************************************************/
/* Version Number of this module */

#define SAMPLE_MAJOR	1
#define SAMPLE_MINOR	0

#include <linux/module.h>
#include <linux/kernel.h>	/* printk() 		*/
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>
#include "HwTest.h"

#define SRAM 	((volatile unsigned long*)0xB0005000)

#define	dbgprint	if(Debug_Enable) printk

//Create Proc Dir HwTest
static struct proc_dir_entry *rootdir = NULL;
short init_status[4]={0xff,0xff,0xff,0xff};

//Proc Entry files
static struct proc_dir_entry *KCSfile = NULL;
static struct proc_dir_entry *I2Cfile = NULL;
static struct proc_dir_entry *GPIOfile = NULL;
static struct proc_dir_entry *ADCfile = NULL;
static struct proc_dir_entry *SRAMfile = NULL;

int st_kcs_read(char *buf,char **start,off_t offset,int count,int *eof,void *data)
{
	if(offset > 0)
	{
    	*eof = 1;
    	return 0;
    }
	return sprintf(buf,"%2x\n",init_status[KCS_OFFSET]);
}
int st_iic_read(char *buf,char **start,off_t offset,int count,int *eof,void *data)
{
    if(offset > 0)
    {
        *eof = 1;
        return 0;
    }
    return sprintf(buf,"%2x\n",init_status[IIC_OFFSET]);
}
int st_gpio_read(char *buf,char **start,off_t offset,int count,int *eof,void *data)
{
    if(offset > 0)
    {
        *eof = 1;
        return 0;
    }
    return sprintf(buf,"%2x\n",init_status[GPIO_OFFSET]);
}
int st_adc_read(char *buf,char **start,off_t offset,int count,int *eof,void *data)
{
	if(offset > 0)
	{
    	*eof = 1;
    	return 0;
    }
	return sprintf(buf,"%2x\n",init_status[ADC_OFFSET]);
}

int sram_read(char *buf,char **start,off_t offset,int count,int *eof,void *data)
{
	unsigned long val;
	if(offset > 0)
	{
		*eof = 1;
		return 0;
	}

	/* Read the value from the SRAM address */
	val = *(SRAM);
	return sprintf(buf,"%lx\n",val);
}

int sram_write(struct file *file, const char __user *buffer,unsigned long count, void *data)
{
	char charval[10];
	unsigned long val;
	int len = 0;

	/* Check whether we have the value to be written onto the SRAM
	 * address is a valid value otherwise we initialize the data to
	 * be 0x00 under the SRAM address */
	if(count > 0)
	{
		len = count;
		if(copy_from_user(charval, buffer, len))
                return -EFAULT;
		charval[len] = '\0';
		sscanf(charval, "%lx", &val);
	}
	else 
	{
		/* We must update the len variable value before returning
		 * from this function, otherwise this function call will
		 * endup in a tight loop */
		len = 5;
		val = 0x00;
	}

	/* Write the value onto the SRAM address */
	*(SRAM) = val;
	return len;
}

int write_init_st(int option,int value)
{
    switch(option)
    {
        case KCS_OPTION:
        case IIC_OPTION:
        case GPIO_OPTION:
        case ADC_OPTION:
                    init_status[option-1]=value;
                    return 0;
        default:
                    return -1;
    }
}


struct proc_dir_entry *AddKCSProc(char *Keyname,struct proc_dir_entry *moduledir)
{
	struct proc_dir_entry *proc = NULL;
	proc=create_proc_entry(Keyname,0444,moduledir);
	if(proc == NULL)
		return NULL;
	proc->read_proc=st_kcs_read;
	proc->write_proc=NULL;
	proc->uid = 0;
    proc->gid = 0;
    proc->size = 1;
	return proc;
}
struct proc_dir_entry *AddIICProc(char *Keyname,struct proc_dir_entry *moduledir)
{
    struct proc_dir_entry *proc = NULL;
    proc=create_proc_entry(Keyname,0444,moduledir);
    if(proc == NULL)
		return NULL;
    proc->read_proc=st_iic_read;
    proc->write_proc=NULL;
    proc->uid = 0;
    proc->gid = 0;
    proc->size = 1;
    return proc;
}
struct proc_dir_entry *AddGPIOProc(char *Keyname,struct proc_dir_entry *moduledir)
{
    struct proc_dir_entry *proc = NULL;
    proc=create_proc_entry(Keyname,0444,moduledir);
    if(proc == NULL)
		return NULL;
    proc->read_proc=st_gpio_read;
    proc->write_proc=NULL;
    proc->uid = 0;
    proc->gid = 0;
    proc->size = 1;
    return proc;
}
struct proc_dir_entry *AddADCProc(char *Keyname,struct proc_dir_entry *moduledir)
{
    struct proc_dir_entry *proc = NULL;
    proc=create_proc_entry(Keyname,0444,moduledir);
    if(proc == NULL)
		return NULL;
    proc->read_proc=st_adc_read;
    proc->write_proc=NULL;
    proc->uid = 0;
    proc->gid = 0;
    proc->size = 1;
    return proc;
}

struct proc_dir_entry *AddSRAMProc(char *Keyname,struct proc_dir_entry *moduledir)
{
    struct proc_dir_entry *proc = NULL;
    proc=create_proc_entry(Keyname,0444,moduledir);
    if(proc == NULL)
		return NULL;
	proc->read_proc=sram_read;
    proc->write_proc=sram_write;
    proc->uid = 0;
    proc->gid = 0;
    proc->size = 1;
    return proc;
}

int init_test_module(void)
{
	
	printk(KERN_ALERT "HwTest Module 1.0 \n");
	printk(KERN_ALERT "Copyright (c) 2009 American Megatrends Inc.\n");
	rootdir = proc_mkdir(PROC_DIR,NULL);
	if(rootdir == NULL)
	{
		printk(KERN_ERR "Can't create the proc directory for HwTest module\n");
		goto onerror;
	}
	KCSfile=AddKCSProc(KCS_PROC_FILE,rootdir);
	if(KCSfile == NULL)
	{
		printk(KERN_ERR "Can't create the /proc/HwTest/kcs entry\n");
		goto onerror;
	}
	I2Cfile=AddIICProc(IIC_PROC_FILE,rootdir);
	if(I2Cfile == NULL)
	{
		printk(KERN_ERR "Can't create the /proc/HwTest/iic entry\n");
		goto onerror;
	}
	GPIOfile=AddGPIOProc(GPIO_PROC_FILE,rootdir);
	if(GPIOfile == NULL)
	{
		printk(KERN_ERR "Can't create the /proc/HwTest/gpio entry\n");
		goto onerror;
	}
	ADCfile=AddADCProc(ADC_PROC_FILE,rootdir);
	if(ADCfile == NULL)
	{
		printk(KERN_ERR "Can't create the /proc/HwTest/adc entry\n");
		goto onerror;
	}
	SRAMfile=AddSRAMProc(SRAM_PROC_FILE,rootdir);
	if(SRAMfile == NULL)
	{
		printk(KERN_ERR "Can't create the /proc/HwTest/sram entry\n");
		goto onerror;
	}
	printk(KERN_ALERT "Hw Test Module Successfully loaded \n");
	return 0;

onerror:
	cleanup_test_module();
	return 1;
}

void cleanup_test_module(void)
{
	printk(KERN_ALERT "Unloading Hw Test Module...\n");
	remove_proc_entry(KCS_PROC_FILE,rootdir);
	remove_proc_entry(IIC_PROC_FILE,rootdir);
	remove_proc_entry(GPIO_PROC_FILE,rootdir);
	remove_proc_entry(ADC_PROC_FILE,rootdir);
	remove_proc_entry(SRAM_PROC_FILE,rootdir);
	remove_proc_entry(PROC_DIR,NULL);
	printk(KERN_ALERT "Hw Test Module unloaded\n");
	return;
}

EXPORT_SYMBOL_GPL(write_init_st);
module_init(init_test_module);
module_exit(cleanup_test_module);

MODULE_AUTHOR("American Megatrends Inc");
MODULE_DESCRIPTION("This is Hw Test module.");
MODULE_LICENSE("GPL");


