/*-------------------------------------------------------------------------
	Copyright (C) 2008-2009 American Megatrends Inc.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-------------------------------------------------------------------------*/
#ifndef	_I2C_DATA_H
#define	_I2C_DATA_H

#include <linux/i2c.h>
#ifdef ARBITRATION_SUPPORT
	#include "i2c-arbitration.h"
#endif

#define	TRANSFERSIZE			1024
#define	MAX_FIFO_LEN			8

#ifdef CFG_PROJ_I2C_USABLE_BUS
#define	BUS_COUNT			CFG_PROJ_I2C_USABLE_BUS
#else
/* Default - All 6 VSC i2c bus are useable */
#define	BUS_COUNT				6
#endif



#define	I2C_BUS_INVALID			-1
#define	I2C_BUS_0				0
#define	I2C_BUS_1				1
#define	I2C_BUS_2				2
#define	I2C_BUS_3				3
#define	I2C_BUS_4				4
#define	I2C_BUS_5				5



#define	SEGMENT_COUNT			10


#define	DEFAULT_SLAVE_0			0x10
#define	DEFAULT_SLAVE_1			0x10
#define	DEFAULT_SLAVE_2			0x10
#define	DEFAULT_SLAVE_3			0x10
#define	DEFAULT_SLAVE_4			0x10
#define	DEFAULT_SLAVE_5			0x10

#define	STATE_IDLE					0
#define	STATE_MASTER_XMIT			1
#define	STATE_MASTER_RECV			2
#define	STATE_SLAVE					3
#define STATE_WAIT_FOR_STOP			4
#define STATE_WAIT_FOR_STOP_AFTER_NACK	5
#define STATE_MASTER_XMIT_RECV		6

#define MAX_NUM_7BIT_SLAVE_ADDR		128
#define MAX_COMB_ADDR				(MAX_NUM_7BIT_SLAVE_ADDR + 8)
#define MAX_DEV_STAT_INDEX			(MAX_NUM_7BIT_SLAVE_ADDR-1)


#define MAX_NON_COLLISION_DEV		20
//#define MAX_DUPLICATE_SET_CNT		4
//#define BBA_BUS_WITH_MUX			2
//#define MAX_DEV_INDEX				MAX_NON_COLLISION_DEV + MAX_DUPLICATE_SET_CNT * SEGMENT_COUNT * BBA_BUS_WITH_MUX // 84 index entry
//#define MAX_DEV_INDEX	10 + 8*4
#define INVALID 					0
#define VALID						1

//#define MAX_INDEX		40

struct i2c_vsc_data
{
	/* Buffer used for Master Transfer */
	unsigned char TX_data[TRANSFERSIZE];
	int TX_len;
	volatile int TX_index;

	/* Buffer used for Master Receive */
	unsigned char MasterRX_data[TRANSFERSIZE];
	int MasterRX_len;
	volatile int MasterRX_index;

/* JM: This driver supports master mode only. No need for slave mode support. */
#if 0
	unsigned char Linear_SlaveRX_data[TRANSFERSIZE];
	int Linear_SlaveRX_len;
	int Linear_SlaveRX_index;

	unsigned char SlaveRX_data[MAX_FIFO_LEN][TRANSFERSIZE];
	int SlaveRX_len[MAX_FIFO_LEN];
	int SlaveRX_index[MAX_FIFO_LEN];

	int SlaveRX_Writer;
	int SlaveRX_Reader;
	int SlaveRX_Entries;
#endif

	spinlock_t data_lock;
	volatile u32 op_status;
	int state;
	wait_queue_head_t vsc_wait;
	unsigned int bus_recovery_cnt;
	unsigned int bus_busy_cnt;
	unsigned char dev_fail_cnt[MAX_NUM_7BIT_SLAVE_ADDR];
	int scan;
	unsigned char slave_addr;
	unsigned int int_cnt;
};

extern int i2c_vsc_add_bus(struct i2c_adapter *);
extern int i2c_vsc_del_bus(struct i2c_adapter *);
extern void i2c_init_internal_data(int	bus);
extern void i2c_init_status_data(int	bus);
extern u8 i2c_vsc_get_dev_stat_index(int bus, u8 slave_addr, u32 seg_num);
extern struct i2c_vsc_data vsc_data_ptr[BUS_COUNT];


//******************************************************I2C STAT ***********************************************/

/* I2C Ioctl Numbers */
#define PASS_ERROR_BIT_TO_I2C_DRV					0x757 /* Pass support fpga error bit to i2c driver */
#define SCAN_DEVICE									0x759
#define CREATE_SCANNED_DEV_SYS_FILES				0x760
#define ADD_DEV_SYS_FILE							0x761
#define DELET_DEV_SYS_FILE							0x762
#define CLEAR_HASH_TABLE							0x763
#define REMOVE_PREVIOUS_SCANNED_DEV_SYS_FILES		0x764
#define I2C_RESET_FAN								0x765
#define CS_FAN_RESET_0								(1<<18)
#define CS_FAN_RESET_1								(1<<19)
#define CS_FAN_RESET_2								(1<<20)
#define CS_ON									(~0)
#define CS_OFF									0
#define FAN1_SEG								4
#define FAN2_SEG								5
#define FAN3_SEG								6


#define HASH_INDEX_VACANT		-1
#define HASH_INDEX_OCCUPIED		0x00


typedef struct { 
	int bus_num;
	unsigned char mux_chan; 
	unsigned char mux_slave;
	unsigned char dev_slave;
} i2c_info_t; 


/* Shared Bus for Bus "2" */
typedef struct {
	int l_rel_short_timeout;
	int l_rel_long_timeout;
	int l_rm_grant_timeout;
	int l_rm_rel_timeout;
	int l_serial_exchange_error;
	int r_rel_short_timeout;
	int r_rel_long_timeout;
	int r_loc_grant_timeout;
	int r_loc_rel_timeout;
	int r_serial_exchange_error;
} sharedbus_stats_t;


typedef struct {
	unsigned long counter;
	int present;
	int activity;
	int mode;
	int xmit_cnt;
	int recv_cnt;
	int recovery_cnt;
	int busy_cnt;
	int conseq_busy_cnt;
	int failure_cnt;
	int conseq_fail_cnt;
	int nack_cnt;
	unsigned char mux_slave;
	unsigned char mux_chan;
	unsigned char dev_slave_addr;
	unsigned char sys_filename[20];
	struct kobj_attribute dev_attribute;
} dev_status_t;
 

typedef struct {
	int collision_cnt;
	unsigned char dev_stat_index_base; 
}hash_table_t;


typedef struct {
	int TX_index;
	int MasterRX_index;
	unsigned int int_cnt;
	unsigned int bus_recovery_cnt;
	unsigned int bus_busy_cnt;
	unsigned int bus_conseq_busy_cnt;
	unsigned int bus_fail_cnt;
	unsigned long scan_counter;
	unsigned char dev_index_cnt;
	unsigned char duplicate_set_cnt;
	hash_table_t hash_table[MAX_COMB_ADDR];
	dev_status_t dev_stat[MAX_NUM_7BIT_SLAVE_ADDR];
} i2c_bus_status_t;



extern i2c_bus_status_t i2c_bus_status_ptr[BUS_COUNT];
extern sharedbus_stats_t g_sharedbus_stats[SEGMENT_COUNT];
extern unsigned int g_mux_chan[BUS_COUNT];


#endif

