/***************************************************************************************
 *
 *  Kernel debug driver shall provide interface to dump , read and write VSC  Registers
 *	
 *****************************************************************************************/


#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/poll.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/wait.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/cacheflush.h>
#include <asm/ioctl.h>
#include <asm/mipsregs.h>

#include "kdebug.h"

/*****Prototye Functions*************************/
static  int  kdbg_init (void );
static void  kdbg_exit (void);
static  int  kdbg_open (struct inode *, struct file *);
static  int  kdbg_release (struct inode *, struct file *);
static  int  kdbg_read (struct file *, char *, size_t, loff_t *);
static  long  kdbg_ioctl ( struct file *, uint, ulong);

static struct file_operations kdbg_ops;
//static void  *kdbg_dev;

uint32_t kdbg_base = 0;
int  kdbg_size = 0; 

/*iowrite ****/
inline void  kdbg_write_reg(unsigned long addr, unsigned long val)
{
        *((volatile uint32_t*)addr) = val;
}

/*********ioread*************/
inline unsigned long  kdbg_read_reg(unsigned long addr )
{
        return ((*(volatile uint32_t*)addr));
}

#define writeByte(addr,val)        *((volatile unsigned char *)(addr)) = val
#define readByte(addr)             *((volatile unsigned char *)(addr))

#if 0

Note : This will always fail at request_mem_region . These regions are already registered by the kernel when the orginal driver is loaded .
We cannot access it again through kernel.  

static int getKernMem(uint32_t base, int size )  
{
	int err; 		
	void *dev = &kdbg_dev; 
	kdbg_base = base;
	kdbg_size = size ;
	
	/* Remap registers */
	if ((dev = ioremap_nocache(kdbg_base,  kdbg_size)) == NULL) 
	{
		printk("failed to map IO space %08x-%08x to memory",kdbg_base, kdbg_base +  kdbg_size - 1);
		return -EIO;
	}

	/* check  memory region */
	if ((err = check_mem_region(kdbg_base, kdbg_size)) < 0)  
	{
		printk("IO space %08x-%08x already in use (err %d)",kdbg_base, kdbg_base +  kdbg_size - 1, err);
		/* Character Device unmapping   */ 
		if (dev) 
		{
			iounmap(dev); 
			dev = NULL;
		}
		return -EBUSY;
	}

	/* request for the memory region */	
	if (request_mem_region(kdbg_base, kdbg_size, KDBG_NAME )  == NULL) 
	{ 
		printk("IO space %08x-%08x spi request mem region failed (err %d)",kdbg_base, kdbg_base +  kdbg_size - 1, err);
		/* Character Device unmapping  */
                if (dev)
                {
                        iounmap(dev);
                        dev = NULL;
                }
		return  -ENODEV;
	}
	
	printk ( "KDBG Driver : getKernMem  \n");
	return 0; 
}


static int ReleaseKernMem(uint32_t kdbg_base, int size)
{
	
	void *dev = &kdbg_dev; 

	/* release the memory occupied */
	release_mem_region(kdbg_base, kdbg_size); 

	/* Character Device unmapping   */ 
	if (dev) 
	{
		iounmap(dev); 
		dev = NULL;
	}

	printk ( "KDBG Driver : ReleaseKernMem \n");
	return 0;
}
  
#endif

/*
 This function reads registers directly without kernel memory map. We cannot again read the same registers already used by the 
orginal driver . Since this driver is Debug driver , we read these regitsers directly without Kernel Memory Map   

*/
 
void 
kdbg_dump_reg(unsigned long addr, int dumpSize)
{
	int i;
	kdbg_base = addr;
        kdbg_size = dumpSize;
	
	//getKernMem(kdbg_base, kdbg_size);
	
	for (i = 0; i < dumpSize ; )
	{
	 	if(( i % 0x10) == 0)
		{
			printk ("\n %x: ", (kdbg_base + i));
		}
		printk("%08lx ",(kdbg_read_reg(kdbg_base +i)));
		i = i + 4; 
	} 
	
	//ReleaseKernMem(kdbg_base, kdbg_size);
	
	return;       
}

#define DISPLAY_CP0(x) printk("C0 Register %20s = 0x%08x\n",#x,__read_32bit_c0_register(x, 0))

void
kdbg_dump_coproc_regs(void)
{
	DISPLAY_CP0( CP0_INDEX );
	DISPLAY_CP0( CP0_RANDOM );
	DISPLAY_CP0( CP0_ENTRYLO0 );
	DISPLAY_CP0( CP0_ENTRYLO1 );
	DISPLAY_CP0( CP0_CONTEXT );
	DISPLAY_CP0( CP0_PAGEMASK );
	DISPLAY_CP0( CP0_WIRED );
	DISPLAY_CP0( CP0_HWRENA );
	DISPLAY_CP0( CP0_BADVADDR );
	DISPLAY_CP0( CP0_COUNT );
	DISPLAY_CP0( CP0_ENTRYHI );
	DISPLAY_CP0( CP0_COMPARE );
	DISPLAY_CP0( CP0_STATUS );
	DISPLAY_CP0( CP0_CAUSE );
	DISPLAY_CP0( CP0_EPC );
	DISPLAY_CP0( CP0_PRID );
	DISPLAY_CP0( CP0_CONFIG );
	DISPLAY_CP0( CP0_LLADDR );
	DISPLAY_CP0( CP0_WATCHLO );
	DISPLAY_CP0( CP0_WATCHHI );
	DISPLAY_CP0( CP0_DEBUG );
	DISPLAY_CP0( CP0_DEBUG_EXE_PC );
	DISPLAY_CP0( CP0_PERFORMANCE );
	DISPLAY_CP0( CP0_ECC );
	DISPLAY_CP0( CP0_TAGLO );
	DISPLAY_CP0( CP0_TAGHI );
	DISPLAY_CP0( CP0_ERROREPC );
	DISPLAY_CP0( CP0_DESAVE );
}

int __init  
kdbg_init(void)
{
	int  err;

	memset(&kdbg_ops,0,sizeof(kdbg_ops));
	kdbg_ops.unlocked_ioctl   = kdbg_ioctl;
	kdbg_ops.open    = kdbg_open;
	kdbg_ops.release = kdbg_release;
	kdbg_ops.read    = kdbg_read;
	kdbg_ops.write   = NULL;
	kdbg_ops.owner   = THIS_MODULE;

	/*********Register Device ********************/

	if ((err = register_chrdev(KDBG_MAJOR_NO, KDBG_NAME, &kdbg_ops)) < 0)
	{
		printk ("failed to register device (err %d)\n", err);
		return -ENODEV;
	}
	printk( KERN_ALERT  "KDBG Module was sucessully Registered \n");
	return 0;
}

	
static void  
kdbg_exit(void)
{
	/* unregister the device */
	unregister_chrdev(KDBG_MAJOR_NO, KDBG_NAME);

	printk ( KERN_ALERT  "KDBG Driver is unloaded sucessfully.\n");
	return;
}

static  int
kdbg_open (struct inode *inode, struct file *file)
{
	file->f_op =&kdbg_ops;
      	return 0;
}

static  int 
kdbg_release (struct inode *inode,struct file *file)
{
	return 0;
}

static  int  
kdbg_read (struct file *file, char *buffer,size_t size, loff_t *offset)
{
	return 0;
}


static long  
kdbg_ioctl (struct file *file, uint cmd , ulong arg)
{
	kdbg_ioctl_t dbgdata;

        switch (cmd)
        {
		case KDBG_READ_WORD:
			if (copy_from_user((void*)&dbgdata, (void*)arg, sizeof(kdbg_ioctl_t))) 
			{
				return -EFAULT;
			}

			dbgdata.wordData = kdbg_read_reg(dbgdata.addr);

			if (copy_to_user((void*)arg, (void*)&dbgdata, sizeof(kdbg_ioctl_t))) 
			{
				return -EFAULT;
			}
                        break;

		case KDBG_WRITE_WORD:

        		if (copy_from_user((void*)&dbgdata, (void*)arg, sizeof(kdbg_ioctl_t))) 
			{
				return -EFAULT;
			}

			kdbg_write_reg(dbgdata.addr, dbgdata.wordData);
			break;


	        case KDBG_READ_BYTE:
			if (copy_from_user((void*)&dbgdata, (void*)arg, sizeof(kdbg_ioctl_t))) 
			{
				return -EFAULT;
			}

			dbgdata.byteData = readByte(dbgdata.addr);

			if (copy_to_user((void*)arg, (void*)&dbgdata, sizeof(kdbg_ioctl_t))) 
			{
				return -EFAULT;
			}
                        break;

		case KDBG_WRITE_BYTE:
        		if (copy_from_user((void*)&dbgdata, (void*)arg, sizeof(kdbg_ioctl_t))) 
			{
				return -EFAULT;
			}

			writeByte(dbgdata.addr, dbgdata.byteData);
			break;	
	
		case KDBG_DUMP_REG:

        		if (copy_from_user((void*)&dbgdata, (void*)arg, sizeof(kdbg_ioctl_t))) 
			{
				return -EFAULT;
			}
			kdbg_dump_reg(dbgdata.addr, dbgdata.dumpSize);
			break;

		case KDBG_DUMP_COPROC_REG:
			kdbg_dump_coproc_regs();
			break;

		default:
			
			break;
	}
	return 0;
}

module_init(kdbg_init);
module_exit(kdbg_exit);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("VSC452 KERNEL DEBUG Driver");


	


